%
% @file    	trajectory_generator.m
% @author  	Luca Fanni lfanni@uniss.it
% @author	Claudio Rubattu crubattu@uniss.it
% @author  	Francesca Palumbo fpalumbo@uniss.it
%
% @brief   	matlab script DLS for the calculation 
%			of the Classic DLS trajectory.
%
% Copyright (c) 2019, Università degli Studi di Sassari
%
% Redistribution and use in source and binary forms, with or without modification, 
% are permitted provided that the following conditions are met:
%
% 1. Redistributions of source code must retain the above copyright notice, this 
% list of conditions and the following disclaimer.
%
% 2. Redistributions in binary form must reproduce the above copyright notice, 
% this list of conditions and the following disclaimer in the documentation and/or 
% other materials provided with the distribution.
%
% 3. Neither the name of the copyright holder nor the names of its contributors may
% be used to endorse or promote products derived from this software without specific 
% prior written permission.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
% EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
% SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
% OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
% HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
% TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
% EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%

function [ThetaOut] = DLS(iterations,lambda,W_pos,thetaIn1,thetaIn2,thetaIn3,thetaIn4)


t1=thetaIn1;
t2=thetaIn2;
t3=thetaIn3;
t4=thetaIn4;
pxr = 5*cos(t1)*(3*sin(t2 - t3) - 3*sin(t3 - t2 + t4) + 3*cos(t2) + sin(t2));
pyr = 5*sin(t1)*(3*sin(t2 - t3) - 3*sin(t3 - t2 + t4) + 3*cos(t2) + sin(t2));
pzr = -15*cos(t2 - t3) - 15*cos(t3 - t2 + t4) - 15.8114*cos(t2 + 1.2490);


position_request=[(W_pos(1)-pxr)./(iterations+1);(W_pos(2)-pyr)./(iterations+1);(W_pos(3)-pzr)./(iterations+1);0;0;0];

theta1=thetaIn1;
theta2=thetaIn2;
theta3=thetaIn3;
theta4=thetaIn4;

for k=1:iterations
 
    t1=theta1;
    t2=theta2;
    t3=theta3;
    t4=theta4;
    
    J=[ -5*sin(t1)*(3*sin(t2 - t3) - 3*sin(t3 - t2 + t4) + 3*cos(t2) + sin(t2)), 5*cos(t1)*(3*cos(t2 - t3) + 3*cos(t3 - t2 + t4) + cos(t2) - 3*sin(t2)), -5*cos(t1)*(3*cos(t2 - t3) + 3*cos(t3 - t2 + t4)), -15*cos(t3 - t2 + t4)*cos(t1);
         5*cos(t1)*(3*sin(t2 - t3) - 3*sin(t3 - t2 + t4) + 3*cos(t2) + sin(t2)), 5*sin(t1)*(3*cos(t2 - t3) + 3*cos(t3 - t2 + t4) + cos(t2) - 3*sin(t2)), -5*sin(t1)*(3*cos(t2 - t3) + 3*cos(t3 - t2 + t4)), -15*cos(t3 - t2 + t4)*sin(t1);
                                                                              0,      15*sin(t2 - t3) - 15*sin(t3 - t2 + t4) + 15.8114*sin(t2 + 1.2490),            15*sin(t3 - t2 + t4) - 15*sin(t2 - t3),          15*sin(t3 - t2 + t4);
                                                                              0,                                                                sin(t1),                                          -sin(t1),                      -sin(t1);
                                                                              0,                                                               -cos(t1),                                           cos(t1),                       cos(t1);
                                                                              1,                                                                      0,                                                 0,                             0];
 

    
    
    J2=J'*J+lambda*lambda*eye(4);
    [cofac,deter]=cof3(J2);
    Jpsinv=(cofac'/deter)*J';   
    thetar= Jpsinv*position_request;
    
    thetar1=theta1+thetar(1,1);
    thetar2=theta2+thetar(2,1);
    thetar3=theta3+thetar(3,1);
    thetar4=theta4+thetar(4,1);

    
theta1=thetar1;
theta2=thetar2;
theta3=thetar3;
theta4=thetar4;

end 

thetaOut1=theta1;
thetaOut2=theta2;
thetaOut3=theta3;
thetaOut4=theta4;

ThetaOut=[thetaOut1,-thetaOut2+pi/2,-thetaOut3,-thetaOut4];