%
% @file    	trajectory_generator.m
% @author  	Luca Fanni lfanni@uniss.it
% @author	Claudio Rubattu crubattu@uniss.it
% @author  	Francesca Palumbo fpalumbo@uniss.it
%
% @brief    matlab script to compute the cofactor matrix.
%
% Copyright (c) 2019, Università degli Studi di Sassari
%
% Redistribution and use in source and binary forms, with or without modification, 
% are permitted provided that the following conditions are met:
%
% 1. Redistributions of source code must retain the above copyright notice, this 
% list of conditions and the following disclaimer.
%
% 2. Redistributions in binary form must reproduce the above copyright notice, 
% this list of conditions and the following disclaimer in the documentation and/or 
% other materials provided with the distribution.
%
% 3. Neither the name of the copyright holder nor the names of its contributors may
% be used to endorse or promote products derived from this software without specific 
% prior written permission.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
% EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
% SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
% OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
% HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
% TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
% EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%

function [cofac,deter] = cof3(J2)

cofac=zeros(4);

min1a=(J2(1,3)*J2(2,4))-(J2(2,3)*J2(1,4));
min1b=(J2(2,3)*J2(3,4))-(J2(3,3)*J2(2,4));
min1c=(J2(3,3)*J2(4,4))-(J2(4,3)*J2(3,4));
min1d=(J2(1,3)*J2(3,4))-(J2(3,3)*J2(1,4));
min1e=(J2(2,3)*J2(4,4))-(J2(4,3)*J2(2,4));
min1f=(J2(1,3)*J2(4,4))-(J2(4,3)*J2(1,4));

min2a=(J2(1,1)*J2(2,2))-(J2(2,1)*J2(1,2));
min2b=(J2(2,1)*J2(3,2))-(J2(3,1)*J2(2,2));
min2c=(J2(3,1)*J2(4,2))-(J2(4,1)*J2(3,2));
min2d=(J2(1,1)*J2(3,2))-(J2(3,1)*J2(1,2));
min2e=(J2(2,1)*J2(4,2))-(J2(4,1)*J2(2,2));
min2f=(J2(1,1)*J2(4,2))-(J2(4,1)*J2(1,2));

cofac(1,1)=((min1c*J2(2,2))+(min1b*J2(4,2)))-(min1e*J2(3,2));
cofac(2,1)=(min1f*J2(3,2))-((min1d*J2(4,2))+(min1c*J2(1,2)));
cofac(3,1)=((min1e*J2(1,2))+(min1a*J2(4,2)))-(min1f*J2(2,2));
cofac(4,1)=(min1d*J2(2,2))-((min1a*J2(3,2))+(min1b*J2(1,2)));
cofac(1,2)=(min1e*J2(3,1))-((min1b*J2(4,1))+(min1c*J2(2,1)));
cofac(2,2)=((min1c*J2(1,1))+(min1d*J2(4,1)))-(min1f*J2(3,1));
cofac(3,2)=(min1f*J2(2,1))-((min1a*J2(4,1))+(min1e*J2(1,1)));
cofac(4,2)=((min1b*J2(1,1))+(min1a*J2(3,1)))-(min1d*J2(2,1));

cofac(1,3)=((min2c*J2(2,4))+(min2b*J2(4,4)))-(min2e*J2(3,4));
cofac(2,3)=(min2f*J2(3,4))-((min2d*J2(4,4))+(min2c*J2(1,4)));
cofac(3,3)=((min2e*J2(1,4))+(min2a*J2(4,4)))-(min2f*J2(2,4));
cofac(4,3)=(min2d*J2(2,4))-((min2a*J2(3,4))+(min2b*J2(1,4)));
cofac(1,4)=(min2e*J2(3,3))-((min2b*J2(4,3))+(min2c*J2(2,3)));
cofac(2,4)=((min2c*J2(1,3))+(min2d*J2(4,3)))-(min2f*J2(3,3));
cofac(3,4)=(min2f*J2(2,3))-((min2a*J2(4,3))+(min2e*J2(1,3)));
cofac(4,4)=((min2b*J2(1,3))+(min2a*J2(3,3)))-(min2d*J2(2,3));

deter=J2(1,1)*cofac(1,1)+J2(1,2)*cofac(1,2)+J2(1,3)*cofac(1,3)+J2(1,4)*cofac(1,4);

